function Hh = dd_frest(ud, yd, z, n) 
T = n + 1; t = (1:T)'; m = size(ud, 2); p = size(yd, 2);

%% preprocessing by low-rank approximation
H = [moshank(ud, T); moshank(yd, T)];
[U, ~, ~] = svd(H); P = U(:, 1:m * T + n); 

for k = 1:length(z)
  %% form and solve the system of equations
  A  = [[zeros(m * T, p); -kron(z(k) .^ t, eye(p))] P];
  hg = A \ [kron(z(k) .^ t, eye(m)); zeros(p * T, m)]; 
  Hh(:, :, k) = hg(1:p, :); 
end

%% ====== auxiliary functions ====== 

function H = moshank(w, i)
if iscell(w)
  N = length(w); H = []; for k = 1:N, H = [H blkhank(w{k}, i)]; end
else
  H = blkhank(w, i);
end

function H = blkhank(w, i, j)
if length(size(w)) == 3 
  [q, N, T]  = size(w);
  if nargin < 3 | isempty(j), j = T - i + 1; end
  if j <= 0, error('Not enough data.'), end
  H = zeros(i * q, j * N);
  for ii = 1:i
    for jj = 1:j
      H(((ii - 1) * q + 1):(ii * q), ...
        ((jj - 1) * N + 1):(jj * N)) = w(: ,:, ii + jj - 1);
    end
  end
else
  [q, T] = size(w); if T < q, w = w'; [q, T] = size(w); end
  if nargin < 3 | isempty(j), j = T - i + 1; end
  if j <= 0, error('Not enough data.'), end
  H = zeros(i * q, j); 
  for ii = 1:i
    H(((ii - 1) * q + 1):(ii * q), :) = w(:, ii:(ii + j - 1));
  end
end